@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -504,9 +504,9 @@ static void user_termination_cb(enum stk_agent_result result, void *user_data)
 	send_simple_response(stk, STK_RESULT_TYPE_USER_TERMINATED);
 }
 
-static void stk_alpha_id_set(struct ofono_stk *stk,
-		const char *text, const struct stk_text_attribute *attr,
-		const struct stk_icon_id *icon)
+static gboolean stk_alpha_id_set(struct ofono_stk *stk,
+			const char *text, const struct stk_text_attribute *attr,
+			const struct stk_icon_id *icon)
 {
 	char *alpha = dbus_apply_text_attributes(text, attr);
 
@@ -516,7 +516,7 @@ static void stk_alpha_id_set(struct ofono_stk *stk,
 	 * better idea is found out.
 	 */
 	if (alpha == NULL)
-		return;
+		return FALSE;
 
 	if (stk->respond_on_exit)
 		stk_agent_display_action(stk->current_agent, alpha, icon,
@@ -525,6 +525,8 @@ static void stk_alpha_id_set(struct ofono_stk *stk,
 		stk_agent_display_action_info(stk->current_agent, alpha, icon);
 
 	g_free(alpha);
+
+	return TRUE;
 }
 
 static void stk_alpha_id_unset(struct ofono_stk *stk)
@@ -1790,6 +1792,45 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 		stk_command_cb(&error, stk);
 }
 
+static void confirm_handled_call_cb(enum stk_agent_result result,
+					gboolean confirm, void *user_data)
+{
+	struct ofono_stk *stk = user_data;
+	const struct stk_command_setup_call *sc =
+					&stk->pending_cmd->setup_call;
+	struct ofono_voicecall *vc = NULL;
+	struct ofono_atom *vc_atom;
+
+	if (stk->driver->user_confirmation == NULL)
+		goto out;
+
+	if (result != STK_AGENT_RESULT_OK) {
+		stk->driver->user_confirmation(stk, FALSE);
+		goto out;
+	}
+
+	stk->driver->user_confirmation(stk, confirm);
+
+	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
+						OFONO_ATOM_TYPE_VOICECALL);
+	if (vc_atom)
+		vc = __ofono_atom_get_data(vc_atom);
+
+	if (vc == NULL)
+		goto out;
+
+	__ofono_voicecall_set_alpha_and_icon_id(vc, sc->addr.number,
+						sc->addr.ton_npi,
+						sc->alpha_id_call_setup,
+						sc->icon_id_call_setup.id);
+
+	return;
+
+out:
+	stk_command_free(stk->pending_cmd);
+	stk->pending_cmd = NULL;
+}
+
 static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 						struct stk_response *rsp,
 						struct ofono_stk *stk)
@@ -2606,6 +2647,54 @@ static gboolean handle_command_launch_browser(const struct stk_command *cmd,
 	return FALSE;
 }
 
+static void setup_call_handled_cancel(struct ofono_stk *stk)
+{
+	struct ofono_voicecall *vc = NULL;
+	struct ofono_atom *vc_atom;
+
+	vc_atom = __ofono_modem_find_atom(
+				__ofono_atom_get_modem(stk->atom),
+				OFONO_ATOM_TYPE_VOICECALL);
+	if (vc_atom)
+		vc = __ofono_atom_get_data(vc_atom);
+
+	if (vc != NULL)
+		__ofono_voicecall_clear_alpha_and_icon_id(vc);
+}
+
+static gboolean handle_setup_call_confirmation_req(struct stk_command *cmd,
+						struct ofono_stk *stk)
+{
+	const struct stk_command_setup_call *sc = &cmd->setup_call;
+	int err;
+	char *alpha_id = dbus_apply_text_attributes(
+					sc->alpha_id_usr_cfm ?
+					sc->alpha_id_usr_cfm : "",
+					&sc->text_attr_usr_cfm);
+	if (alpha_id == NULL)
+		goto out;
+
+	err = stk_agent_confirm_call(stk->current_agent, alpha_id,
+					&sc->icon_id_usr_cfm,
+					confirm_handled_call_cb,
+					stk, NULL,
+					stk->timeout * 1000);
+	g_free(alpha_id);
+
+	if (err < 0)
+		goto out;
+
+	stk->cancel_cmd = setup_call_handled_cancel;
+
+	return TRUE;
+
+out:
+	if (stk->driver->user_confirmation)
+		stk->driver->user_confirmation(stk, FALSE);
+
+	return FALSE;
+}
+
 static void stk_proactive_command_cancel(struct ofono_stk *stk)
 {
 	if (stk->immediate_response)
@@ -2812,12 +2901,23 @@ void ofono_stk_proactive_command_notify(struct ofono_stk *stk,
 		stk_command_cb(&error, stk);
 }
 
+static gboolean handled_alpha_id_set(struct ofono_stk *stk,
+			const char *text, const struct stk_text_attribute *attr,
+			const struct stk_icon_id *icon)
+{
+	if (stk_alpha_id_set(stk, text, attr, icon) == FALSE)
+		return FALSE;
+
+	stk->cancel_cmd = stk_alpha_id_unset;
+	return TRUE;
+}
+
 void ofono_stk_proactive_command_handled_notify(struct ofono_stk *stk,
 						int length,
 						const unsigned char *pdu)
 {
-	struct stk_command *cmd;
 	struct stk_response dummyrsp;
+	gboolean ok = FALSE;
 
 	/*
 	 * Modems send us the proactive command details and terminal responses
@@ -2826,69 +2926,75 @@ void ofono_stk_proactive_command_handled_notify(struct ofono_stk *stk,
 	 * responses here
 	 */
 	if (length > 0 && pdu[0] == 0x81) {
-		stk_alpha_id_unset(stk);
+		stk_proactive_command_cancel(stk);
 		return;
 	}
 
 	stk_proactive_command_cancel(stk);
 
-	cmd = stk_command_new_from_pdu(pdu, length);
-
-	if (cmd == NULL || cmd->status != STK_PARSE_RESULT_OK) {
-		ofono_error("Can't parse proactive command");
-
-		if (cmd)
-			stk_command_free(cmd);
+	stk->pending_cmd = stk_command_new_from_pdu(pdu, length);
+	if (stk->pending_cmd == NULL)
 		return;
+
+	if (stk->pending_cmd->status != STK_PARSE_RESULT_OK) {
+		ofono_error("Can't parse modem-handled proactive command");
+		ok = FALSE;
+		goto out;
 	}
 
-	DBG("type: %d", cmd->type);
-
-	switch (cmd->type) {
-	case STK_COMMAND_TYPE_MORE_TIME:
-		break;
+	DBG("type: %d", stk->pending_cmd->type);
 
+	switch (stk->pending_cmd->type) {
 	case STK_COMMAND_TYPE_SEND_SMS:
-		stk_alpha_id_set(stk, cmd->send_sms.alpha_id,
-				&cmd->send_sms.text_attr,
-				&cmd->send_sms.icon_id);
+		ok = handled_alpha_id_set(stk,
+					stk->pending_cmd->send_sms.alpha_id,
+					&stk->pending_cmd->send_sms.text_attr,
+					&stk->pending_cmd->send_sms.icon_id);
 		break;
 
 	case STK_COMMAND_TYPE_SETUP_IDLE_MODE_TEXT:
-		handle_command_set_idle_text(cmd, &dummyrsp, stk);
+		handle_command_set_idle_text(stk->pending_cmd, &dummyrsp, stk);
 		break;
 
 	case STK_COMMAND_TYPE_SETUP_MENU:
-		handle_command_set_up_menu(cmd, &dummyrsp, stk);
+		handle_command_set_up_menu(stk->pending_cmd, &dummyrsp, stk);
 		break;
 
 	case STK_COMMAND_TYPE_SETUP_CALL:
-		/* TODO */
+		ok = handle_setup_call_confirmation_req(stk->pending_cmd, stk);
 		break;
 
 	case STK_COMMAND_TYPE_SEND_USSD:
-		stk_alpha_id_set(stk, cmd->send_ussd.alpha_id,
-				&cmd->send_ussd.text_attr,
-				&cmd->send_ussd.icon_id);
+		ok = handled_alpha_id_set(stk,
+					stk->pending_cmd->send_ussd.alpha_id,
+					&stk->pending_cmd->send_ussd.text_attr,
+					&stk->pending_cmd->send_ussd.icon_id);
 		break;
 
 	case STK_COMMAND_TYPE_SEND_SS:
-		stk_alpha_id_set(stk, cmd->send_ss.alpha_id,
-				&cmd->send_ss.text_attr,
-				&cmd->send_ss.icon_id);
+		ok = handled_alpha_id_set(stk,
+					stk->pending_cmd->send_ss.alpha_id,
+					&stk->pending_cmd->send_ss.text_attr,
+					&stk->pending_cmd->send_ss.icon_id);
+		break;
 
 	case STK_COMMAND_TYPE_SEND_DTMF:
-		stk_alpha_id_set(stk, cmd->send_dtmf.alpha_id,
-				&cmd->send_dtmf.text_attr,
-				&cmd->send_dtmf.icon_id);
+		ok = handled_alpha_id_set(stk,
+					stk->pending_cmd->send_dtmf.alpha_id,
+					&stk->pending_cmd->send_dtmf.text_attr,
+					&stk->pending_cmd->send_dtmf.icon_id);
 		break;
 
 	case STK_COMMAND_TYPE_REFRESH:
-		handle_command_refresh(cmd, NULL, stk);
+		handle_command_refresh(stk->pending_cmd, NULL, stk);
 		break;
 	}
 
-	stk_command_free(cmd);
+out:
+	if (ok == FALSE) {
+		stk_command_free(stk->pending_cmd);
+		stk->pending_cmd = NULL;
+	}
 }
 
 int ofono_stk_driver_register(const struct ofono_stk_driver *d)
